import java.sql.*;
import java.time.LocalDate;
import java.util.*;

/**
 * klient, ktery pristupuje k databazi
 * nacte data pro prepocet, ktera preda tride EquationParser
 * data rozdeluje do urovni
 * sumace vypisuje, prepocty zapisuje zpet do databaze
 *
 * @author Petr Holický
 */
public class Client {
    /* parser ktery zpracuje a vypocte zadany vzorecek */
    EquationParser equationParser;

    /* spojeni s databazi */
    Connection connection;

    /* ukladani dat ktera se pocitaji */
    /* nova data */
    static List<HashMap<String, Object>> new_data = new ArrayList<>();
    /* prepoctena data */
    static List<HashMap<String, Object>> recalcedData = new ArrayList<>();
    /* data rozdelena do jobu */
    static HashMap<Key, List<HashMap<String, Object>>> jobs = new HashMap<>();
    /* prepoctene joby */
    static HashMap<Key, HashMap<String, Object>> recalcedJobs = new HashMap<>();
    /* joby rozdeleny do dnu */
    static HashMap<Key, List<HashMap<String, Object>>> daysFromJobs = new HashMap<>();
    /* data rozdeleny do dnu */
    static HashMap<Key, List<HashMap<String, Object>>> daysFromData = new HashMap<>();
    /* prepoctene dny */
    static HashMap<Key, HashMap<String, Object>> recalcedDays = new HashMap<>();
    /* nactene dny pro sumace */
    static HashMap<Integer, List<HashMap<String, Object>>> summs = new HashMap<>();
    /* vypoctene sumace */
    static HashMap<Integer, HashMap<String, Object>> calcedSumms = new HashMap<>();

    /* sleduji zda jednotlive radky dat poterbuji prepocitat */
    static List<Boolean> new_data_finished = new ArrayList<>();
    static HashMap<Key, Boolean> jobs_finished = new HashMap<>();
    static HashMap<Key, Boolean> daysFromJobs_finished = new HashMap<>();
    static HashMap<Integer, Boolean> summ_finished = new HashMap<>();

    /* ukladani pomocnych dat nactenych z databaze */
    static HashMap<Key, HashMap<String, Object>> jobsFromDB = new HashMap<>();
    static HashMap<Key, List<HashMap<String, Object>>> jobDataFromDB = new HashMap<>();
    static HashMap<Key, HashMap<String, Object>> daysFromDB = new HashMap<>();
    static HashMap<Key, List<HashMap<String, Object>>> dayDataFromDB = new HashMap<>();
    /* nactena data z databaze pro sumace */
    static HashMap<Integer, List<HashMap<String, Object>>> jobsFromDB_SUM = new HashMap<>();
    static HashMap<Integer, HashMap<String, Object>> daysFromDB_SUM = new HashMap<>();

    /* konstanty z databaze */
    static HashMap<String, HashMap<String, Object>> constants = new HashMap<>();
    /* vzorecky z databaze */
    HashMap<Integer, List<HashMap<String, Object>>> equations = new HashMap<>();

    /* pocitadlo: kolik vypoctu selhalo */
    static int recalcsFailedCount = 0;

    /* casove rozmezi pro sumace */
    static Timestamp startStamp;
    static Timestamp stopStamp;

    /* promenne ktere udavaji maximalni pocet nactenych radek */
    private final int dataLimit = 5000;
    private final int jobLimit = 10;
    private final int dayLimit = 5;

    /**
     * pripoji se k databazi
     * inicializuje connention na dane spojeni
     * @throws SQLException sql exception
     */
    private void connect() throws SQLException{
        Properties properties = new Properties();
        properties.setProperty("user", "admin");
        properties.setProperty("password", "quest");
        properties.setProperty("sslmode", "disable");

       this.connection = DriverManager.getConnection("jdbc:postgresql://localhost:8812/qdb", properties);
    }

    /**
     * uzavre spojeni s databazi ktere je ulozene v connection
     * @throws SQLException sql exception
     */
    private void closeConnection() throws SQLException{
        connection.close();
    }

    /**
     * nacte data pro prepocet
     * pokud jsou nacteny joby/dny -> jsou nacteny i data ze kterych se pocitaly - urcena za dokoncene
     * data urcena pro prepocet oznaci za nedokoncena
     * @throws SQLException sql exception
     */
    private void selectData() throws SQLException {
        //select dat pro prepocet
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM data WHERE recalc>0 limit "+dataLimit, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    HashMap<String, Object> row = new HashMap<>(columns);
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));
                    }
                    if(!row.isEmpty()) {
                        new_data.add(row);
                        new_data_finished.add(false);
                    }
                }
            }
        }
        System.out.println("DATA SELECTED");
        //select jobu pro prepocet
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM job WHERE recalc>0 limit "+jobLimit, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    HashMap<String, Object> row = new HashMap<>(columns);
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));
                    }
                    if(!row.isEmpty())
                        recalcedJobs.put(new Key((Timestamp) row.get("datetime"), (int) row.get("id")), row);
                }
            }
        }
        //select na data z kterych se pocitaly joby
        if(!recalcedJobs.isEmpty()) {
            for(Key key : recalcedJobs.keySet()) {
                try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM data WHERE data_time='"+key.getData_time()+"' AND id="+key.getId()+" AND recalc=0", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                    try (ResultSet rs = preparedStatement.executeQuery()) {
                        rs.last();
                        int rows = rs.getRow();
                        if(rows <= 0) {
                            System.out.println("Necessary data for job recalculation not found!");
                            System.exit(1);
                        }
                        rs.beforeFirst();
                        ResultSetMetaData md = rs.getMetaData();
                        int columns = md.getColumnCount();
                        while (rs.next()) {
                            HashMap<String, Object> row = new HashMap<>(columns);
                            for (int i = 1; i <= columns; ++i) {
                                row.put(md.getColumnName(i), rs.getObject(i));
                            }
                            if(!row.isEmpty()) {
                                System.out.println(row);
                                new_data.add(row);
                                new_data_finished.add(true);
                                if(jobs.containsKey(key))
                                    jobs.get(key).add(row);
                                else  {
                                    List<HashMap<String, Object>> l = new ArrayList<>();
                                    l.add(row);
                                    jobs.put(key, l);
                                }
                                jobs_finished.put(key, true);
                            }
                        }
                    }
                }
            }
        }
        System.out.println("JOBS SELECTED");
        //select dni pro prepocet
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM day WHERE recalc>0 limit "+dayLimit, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    HashMap<String, Object> row = new HashMap<>(columns);
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));

                    }
                    if(!row.isEmpty())
                        recalcedDays.put(new Key((Timestamp) row.get("datetime"), (int) row.get("id")), row);
                }
            }
        }
        //select na joby a data ze kterych se pocitaly dny
        if(!recalcedDays.isEmpty()) {
            for(Key key : recalcedDays.keySet()) {
                Timestamp data_time = (Timestamp) recalcedDays.get(key).get("data_time");
                try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM job WHERE datetime>='"+key.getData_time()+"' AND data_time<='"+data_time+"' AND id="+key.getId()+" AND recalc=0", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                    try (ResultSet rs = preparedStatement.executeQuery()) {
                        rs.last();
                        int rows = rs.getRow();
                        if(rows <= 0) {
                            System.out.println("Necessary data for job recalculation not found!");
                            System.exit(1);
                        }
                        rs.beforeFirst();
                        ResultSetMetaData md = rs.getMetaData();
                        int columns = md.getColumnCount();
                        while (rs.next()) {
                            HashMap<String, Object> row = new HashMap<>(columns);
                            for (int i = 1; i <= columns; ++i) {
                                row.put(md.getColumnName(i), rs.getObject(i));
                            }
                            if(!row.isEmpty()) {
                                recalcedJobs.put(key, row);
                            }
                        }
                    }
                }
                try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM data WHERE datetime>='"+key.getData_time()+"' AND datetime<='"+data_time+"' AND id="+key.getId()+" AND recalc=0", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                    try (ResultSet rs = preparedStatement.executeQuery()) {
                        rs.last();
                        int rows = rs.getRow();
                        if(rows <= 0) {
                            System.out.println("Necessary data for job recalculation not found!");
                            System.exit(1);
                        }
                        rs.beforeFirst();
                        ResultSetMetaData md = rs.getMetaData();
                        int columns = md.getColumnCount();
                        while (rs.next()) {
                            HashMap<String, Object> row = new HashMap<>(columns);
                            for (int i = 1; i <= columns; ++i) {
                                row.put(md.getColumnName(i), rs.getObject(i));
                            }
                            if(!row.isEmpty()) {
                                new_data.add(row);
                                new_data_finished.add(true);
                                if(jobs.containsKey(key))
                                    jobs.get(key).add(row);
                                else  {
                                    List<HashMap<String, Object>> l = new ArrayList<>();
                                    l.add(row);
                                    jobs.put(key, l);
                                }
                                jobs_finished.put(key, true);
                            }
                        }
                    }
                }
            }
        }
        System.out.println("DATA SELECTED");
    }

    /**
     * nacte dny do new_data v zadanem rozmezi casu
     * sumace
     * @throws SQLException sql exception
     */
    private void selectDaysBetween() throws SQLException {
        List<HashMap<String, Object>> list = new ArrayList<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM day WHERE datetime >= '"+startStamp+"' AND data_time <= '"+stopStamp+"'", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    HashMap<String, Object> row = new HashMap<>(columns);
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));

                    }
                    list.add(row);
                }
            }
        }
        new_data = list;
    }

    /**
     * nacte vzorecky z databaze pro dane zarizeni
     * ulozi do equations pro opetovne pouziti
     * @param id id zarizeni
     * @return list nactenych vzorecku
     * @throws SQLException sql exception
     */
    private List<HashMap<String, Object>> getEquations(int id) throws SQLException {
        if(equations.containsKey(id)) {
            return equations.get(id);
        }
        List<HashMap<String, Object>> list = new ArrayList<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM equations WHERE id="+id, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    HashMap<String, Object> row = new HashMap<>(columns);
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));

                    }
                    list.add(row);
                }
            }
        }
        equations.put(id, list);
        return list;
    }

    /**
     * aplikuje vzorecky urovne 2 na new_data pres equationParser
     * prepoctena data ulozi do recalcedData
     * @throws Exception sql exception
     */
    private void recalcData() throws Exception {
        for(int i = 0; i<new_data.size(); i++) {
            if(new_data_finished.size() > i && new_data_finished.get(i)) {
                continue;
            }
            HashMap<String, Object> row = new_data.get(i);
            String new_conjoined = (String) new_data.get(i).get("conjoined");
            List<HashMap<String, Object>> equations = getEquations((Integer) row.get("id"));
            for(HashMap<String, Object> equation : equations) {
                if((Integer) equation.get("type") == 2) {
                    new_conjoined = equationParser.applyEquation(new_conjoined, equation, i, Level.DATA);
                }
            }
            HashMap<String, Object> new_row = new HashMap<>(row);
            new_row.put("recalc", 0);
            new_row.put("conjoined", new_conjoined);

            if(recalcedData.size() > i)
                recalcedData.set(i, new_row);
            else
                recalcedData.add(i, new_row);
        }
    }

    /**
     * aplikuje vzorecky urovne 4 na jobs pres equationParser
     * prepoctene joby ulozi jako jeden zaznam do recalcedJobs
     * @throws SQLException sql exception
     */
    private void recalcJobs() throws SQLException {
        for(Key key : jobs.keySet()){
            if(jobs_finished.containsKey(key) && jobs_finished.get(key)) {
                continue;
            }
            String new_conjoined = "";
            List<HashMap<String, Object>> job = jobs.get(key);
            List<HashMap<String, Object>> equations = getEquations((Integer) job.get(0).get("id"));
            for(HashMap<String, Object> equation : equations) {
                if((Integer) equation.get("type") == 4) {
                    new_conjoined = equationParser.applyEquation(new_conjoined, equation, key, Level.JOB);
                }
            }
            Timestamp start = (Timestamp) job.get(0).get("datetime");
            Timestamp end = (Timestamp) job.get(0).get("datetime");
            for(HashMap<String, Object> row : job) {
                if(((Timestamp) row.get("datetime")).before(start)) {
                    start = (Timestamp) row.get("datetime");
                }
                if(((Timestamp) row.get("datetime")).after(end)) {
                    end = (Timestamp) row.get("datetime");
                }
            }

            HashMap<String, Object> new_job = new HashMap<>();
            new_job.put("datetime", start);
            new_job.put("data_time", end);
            new_job.put("id", job.get(0).get("id"));
            new_job.put("state", job.get(0).get("state"));
            new_job.put("gps", job.get(0).get("gps"));
            new_job.put("user", job.get(0).get("user"));
            new_job.put("recalc", 0);
            new_job.put("conjoined", new_conjoined);
            recalcedJobs.put(key, new_job);
        }
    }

    /**
     * aplikuje vzorecky urovne 5 na daysFromJobs pres equationParser
     * prepoctene dny ulozi jako jeden zaznam do recalcedDays
     * @throws SQLException sql exception
     */
    private void recalcDays() throws SQLException {
        for(Key key : daysFromJobs.keySet()) {
            if(daysFromJobs_finished.containsKey(key) && daysFromJobs_finished.get(key)) {
                continue;
            }
            String new_conjoined = "";
            List<HashMap<String, Object>> equations = getEquations(key.getId());
            for(HashMap<String, Object> equation : equations) {
                if((Integer) equation.get("type") == 5) {
                    new_conjoined = equationParser.applyEquation(new_conjoined, equation, key, Level.DAY);
                }
            }
            List<HashMap<String, Object>> day = daysFromJobs.get(key);
            Timestamp start = (Timestamp) day.get(0).get("datetime");
            Timestamp end = (Timestamp) day.get(0).get("data_time");
            for(HashMap<String, Object> row : day) {
                if(((Timestamp) row.get("datetime")).before(start)) {
                    start = (Timestamp) row.get("datetime");
                }
                if(((Timestamp) row.get("data_time")).after(end)) {
                    end = (Timestamp) row.get("data_time");
                }
            }
            //vytvoreni jednoho zaznamu ze vsech dat
            HashMap<String, Object> day_data = new HashMap<>();
            day_data.put("datetime", start);
            day_data.put("data_time", end);
            day_data.put("id", key.getId());
            day_data.put("state", day.get(0).get("state"));
            day_data.put("gps", day.get(0).get("gps"));
            day_data.put("user", day.get(0).get("user"));
            day_data.put("recalc", 0);
            day_data.put("conjoined", new_conjoined);
            recalcedDays.put(key, day_data);
        }
    }

    /**
     * aplikuje vzorecky urovne 6 a 7 na summs pres equationParser
     * vypoctene sumace ulozi jako jeden zaznam do calcedSumms
     * @throws SQLException sql exception
     */
    private void calcSumTot() throws SQLException {
        for(int id : summs.keySet()) {
            if(summ_finished.containsKey(id) && summ_finished.get(id)) {
                continue;
            }
            String new_conjoined = "";
            List<HashMap<String, Object>> equations = getEquations(id);
            for(HashMap<String, Object> equation : equations) {
                if((Integer) equation.get("type") == 6 || (Integer) equation.get("type") == 7) {
                    new_conjoined = equationParser.applyEquation(new_conjoined, equation, id, Level.SUM);
                }
            }
            HashMap<String, Object> summ_row = new HashMap<>();
            summ_row.put("datetime", startStamp);
            summ_row.put("data_time", stopStamp);
            summ_row.put("id", id);
            summ_row.put("state", summs.get(id).get(0).get("state"));
            summ_row.put("gps", summs.get(id).get(0).get("gps"));
            summ_row.put("user", summs.get(id).get(0).get("user"));
            summ_row.put("recalc", 0);
            summ_row.put("conjoined", new_conjoined);
            calcedSumms.put(id, summ_row);
        }
    }

    /**
     * prepoctena data z recalcedData rozdeli do jobu a ulozi do jobs
     * job = stejny cas ve sloupci data_time
     */
    private void getJobs() {
        jobs = new HashMap<>();
        List<HashMap<String, Object>> job = new ArrayList<>();
        for (HashMap<String, Object> row : recalcedData) {
            Key key = new Key((Timestamp) row.get("data_time"), (Integer) row.get("id"));
            //job uz existuje -> pridat data
            if(jobs.containsKey(key)) {
               jobs.get(key).add(row);
            }
            //job neexistuje -> vytvorit
            else {
                job.add(row);
                jobs.put(key, job);
                job = new ArrayList<>();
            }
        }
    }

    /**
     * prepoctene joby z recalcedJobs rozdeli do dni a ulozi do daysFromJobs
     * prepoctene data z jobs rozdeli do dni a ulozi do daysFromData
     * klic z jobu je zachovan i pro dny
     * den = job ve stejny den + po definovanem case zlomu dne
     */
    private void getDays() {
        List<HashMap<String, Object>> fromRaw = new ArrayList<>();
        List<HashMap<String, Object>> fromCalced = new ArrayList<>();

        HashMap<Key, Boolean> bannedKeys = new HashMap<>();

        for(Key key1 : recalcedJobs.keySet()) {
            if(bannedKeys.containsKey(key1))
                continue;
            fromCalced.add(recalcedJobs.get(key1));
            fromRaw.addAll(jobs.get(key1));

            bannedKeys.put(key1, true);
            if(bannedKeys.size() == recalcedJobs.size()) {
                daysFromJobs.put(key1, fromCalced);
                daysFromData.put(key1, fromRaw);
                break;
            }

            LocalDate localDate1 = key1.getData_time().toLocalDateTime().toLocalDate();
            for(Key key2 : recalcedJobs.keySet()) {
                if(bannedKeys.containsKey(key2) || key1.equals(key2) || key1.getId() != key2.getId())
                    continue;
                LocalDate localDate2 = key2.getData_time().toLocalDateTime().toLocalDate();
                // cas prelomu dne pro dany stroj
                String timeBreak = this.equationParser.getConstant(String.valueOf(key1.getId()), "WB").get("value").toString();
                StringBuilder wb_sb = new StringBuilder(timeBreak);
                wb_sb.insert(2, ':');
                wb_sb.insert(5, ':');
                String wb_date =  key1.getData_time().toString().split(" ")[0];
                Timestamp wb_ts = Timestamp.valueOf(wb_date+" "+wb_sb);
                Calendar wb_cal = Calendar.getInstance();
                wb_cal.setTime(wb_ts);
                Timestamp ts = new Timestamp(wb_cal.getTime().getTime());

                if(localDate1.isEqual(localDate2) && Timestamp.valueOf(key2.getData_time().toLocalDateTime()).after(ts)) {
                    fromCalced.add(recalcedJobs.get(key2));
                    fromRaw.addAll(jobs.get(key2));

                    bannedKeys.put(key2, true);
                }
            }
            daysFromJobs.put(key1, fromCalced);
            daysFromData.put(key1, fromRaw);

            fromRaw = new ArrayList<>();
            fromCalced = new ArrayList<>();
            if(bannedKeys.size() == recalcedJobs.size())
                break;
        }
    }

    /**
     * rozdely nactene dny v new_data podle id
     * ulozi do summs
     * pro sumace
     */
    private void categorizeById() {
        List<HashMap<String, Object>> day = new ArrayList<>();
        for(HashMap<String, Object> row : new_data) {
            int id = (int) row.get("id");
            if(summs.containsKey(id)) {
                summs.get(id).add(row);
            }
            else {
                day.add(row);
                summs.put(id, day);
                day = new ArrayList<>();
            }
        }
    }

    /**
     * zapise vypoctena data do databáze
     * TATO METODA JE POUZE DOCASNE RESENI
     * JE NUTNO NAHRADIT PRIKAZEM SELECT/DELETE PO VYDANI PODPORY
     * @throws SQLException sql exception
     */
    private void insertToDB() throws SQLException {
        PreparedStatement preparedStatement = connection.prepareStatement("CREATE TABLE datacopy AS (SELECT * FROM data EXCEPT (SELECT * FROM data WHERE recalc>0 limit "+dataLimit+")) timestamp(datetime) PARTITION BY hour;");
        preparedStatement.execute();
        PreparedStatement preparedStatementInsert = connection.prepareStatement("INSERT INTO datacopy VALUES (to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), ?, ?, ?, ?, ?, ?)");
        for (HashMap<String, Object> data : recalcedData) {
            preparedStatementInsert.setObject(1, data.get("datetime").toString());
            preparedStatementInsert.setObject(2, data.get("data_time").toString());
            preparedStatementInsert.setObject(3, data.get("id"));
            preparedStatementInsert.setObject(4, data.get("state"));
            preparedStatementInsert.setObject(5, data.get("user"));
            preparedStatementInsert.setObject(6, data.get("gps"));
            preparedStatementInsert.setObject(7, data.get("recalc"));
            preparedStatementInsert.setObject(8, data.get("conjoined"));
            preparedStatementInsert.addBatch();
        }
        preparedStatementInsert.executeBatch();

        preparedStatement = connection.prepareStatement("CREATE TABLE jobcopy AS (SELECT * FROM job EXCEPT (SELECT * FROM job WHERE recalc>0 limit "+jobLimit+")) timestamp(datetime) PARTITION BY hour;");
        preparedStatement.execute();
        preparedStatementInsert = connection.prepareStatement("INSERT INTO jobcopy VALUES (to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), ?, ?, ?, ?, ?, ?)");
        for (HashMap<String, Object> job : recalcedJobs.values()) {
            preparedStatementInsert.setObject(1, job.get("datetime").toString());
            preparedStatementInsert.setObject(2, job.get("data_time").toString());
            preparedStatementInsert.setObject(3, job.get("id"));
            preparedStatementInsert.setObject(4, job.get("state"));
            preparedStatementInsert.setObject(5, job.get("user"));
            preparedStatementInsert.setObject(6, job.get("gps"));
            preparedStatementInsert.setObject(7, job.get("recalc"));
            preparedStatementInsert.setObject(8, job.get("conjoined"));
            preparedStatementInsert.addBatch();
        }
        preparedStatementInsert.executeBatch();

        preparedStatement = connection.prepareStatement("CREATE TABLE daycopy AS (SELECT * FROM day EXCEPT (SELECT * FROM day WHERE recalc>0 limit "+dayLimit+")) timestamp(datetime) PARTITION BY hour;");
        preparedStatement.execute();
        preparedStatementInsert = connection.prepareStatement("INSERT INTO daycopy VALUES (to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), ?, ?, ?, ?, ?, ?)");
        for (HashMap<String, Object> day : recalcedDays.values()) {
            preparedStatementInsert.setObject(1, day.get("datetime").toString());
            preparedStatementInsert.setObject(2, day.get("data_time").toString());
            preparedStatementInsert.setObject(3, day.get("id"));
            preparedStatementInsert.setObject(4, day.get("state"));
            preparedStatementInsert.setObject(5, day.get("user"));
            preparedStatementInsert.setObject(6, day.get("gps"));
            preparedStatementInsert.setObject(7, day.get("recalc"));
            preparedStatementInsert.setObject(8, day.get("conjoined"));
            preparedStatementInsert.addBatch();
        }
        preparedStatementInsert.executeBatch();

        preparedStatement = connection.prepareStatement("DROP TABLE data;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("RENAME TABLE datacopy TO data;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("DROP TABLE job;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("RENAME TABLE jobcopy TO job;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("DROP TABLE day;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("RENAME TABLE daycopy TO day;");
        preparedStatement.execute();
    }

    /**
     * main metoda
     * pro 0 argumentu provede prepocet dat
     * pro 2 argumenty, ktere jsou casy ve formatu yyyy-MM-dd provede sumace
     * vypocty se opakuji ve smycce dokud se neco pocita
     * @param args vstupni argumenty programu
     * @throws Exception exception
     */
    public static void main(String[] args) throws Exception {
        //summary
        if(args.length == 2) {
            System.out.println("------SUMMARY------");
            if(args[0].matches("\\d{4}-\\d{2}-\\d{2}") && args[1].matches("\\d{4}-\\d{2}-\\d{2}")) {
                startStamp = Timestamp.valueOf(args[0]+" 00:00:00");
                stopStamp = Timestamp.valueOf(args[1]+" 00:00:00");
                System.out.println("CALCULATING SUMMARY IN RANGE "+startStamp+" TO "+stopStamp);
                Client client = new Client();
                client.connect();
                client.equationParser = new EquationParser(client.connection);
                client.selectDaysBetween();
                System.out.println("DATA ACQUIRED");
                client.categorizeById();
                System.out.println("DATA CATEGORIZED");
                client.calcSumTot();
                System.out.println("SUMMARY CALCULATED");

                int prevUnrecalced = recalcsFailedCount;
                recalcsFailedCount = 0;
                while(true) {
                    System.out.println("NEXT ITERATION");
                    client.calcSumTot();
                    System.out.println("SUMMARY RECALCULATED");
                    if(recalcsFailedCount == 0)
                        break;
                    if(recalcsFailedCount >= prevUnrecalced) {
                        break;
                    } else {
                        prevUnrecalced = recalcsFailedCount;
                        recalcsFailedCount = 0;
                    }
                }
                System.out.println("SUMMARY FINISHED");
                for(int id : calcedSumms.keySet()) {
                    System.out.println(id+": "+calcedSumms.get(id));
                }
                client.closeConnection();
            } else {
                System.out.println("Invalid arguments!\nWrite the start date and end date of the range in yyyy-MM-dd format.");
            }
        }
        //recalc
        else if(args.length == 0) {
            System.out.println("------RECALCULATING------");
            Client client = new Client();
            client.connect();
            client.equationParser = new EquationParser(client.connection);
            System.out.println("CONNECTED");
            client.selectData();
            System.out.println("NEW DATA ACQUIRED");
            client.recalcData();
            System.out.println("DATA RECALCULATED");
            client.getJobs();
            System.out.println("DATA SORTED OUT INTO JOBS");
            client.recalcJobs();
            System.out.println("JOBS RECALCULATED");
            client.getDays();
            System.out.println("JOBS SORTED INTO DAYS");
            client.recalcDays();
            System.out.println("DAYS RECALCULATED");

            int prevUnrecalced = recalcsFailedCount;
            recalcsFailedCount = 0;
            while(true) {
                System.out.println("NEXT ITERATION");
                new_data = new ArrayList<>(recalcedData);
                client.recalcData();
                System.out.println("DATA RECALCULATED");
                client.getJobs();
                client.recalcJobs();
                System.out.println("JOBS RECALCULATED");
                client.getDays();
                client.recalcDays();
                System.out.println("DAYS RECALCULATED");
                if(recalcsFailedCount == 0)
                    break;
                if(recalcsFailedCount >= prevUnrecalced) {
                    break;
                } else {
                    prevUnrecalced = recalcsFailedCount;
                    recalcsFailedCount = 0;
                }
            }
            System.out.println("ALL CALCULATIONS FINISHED");
            /*System.out.println("------new_data------");
            System.out.println(new_data);
            System.out.println("------recalcedData------");
            System.out.println(recalcedData);
            System.out.println("------jobs------");
            jobs.forEach((k,v) -> System.out.println("key: "+k+" value:"+v));
            System.out.println("------recalcedJobs------");
            recalcedJobs.forEach((k,v) -> System.out.println("key: "+k+" value:"+v));
            System.out.println("------daysFromData------");
            daysFromData.forEach((k,v) -> System.out.println("key: "+k+" value:"+v));
            System.out.println("------daysFromJobs------");
            daysFromJobs.forEach((k,v) -> System.out.println("key: "+k+" value:"+v));
            System.out.println("------recalcedDays------");
            recalcedDays.forEach((k,v) -> System.out.println("key: "+k+" value:"+v));
            */
            System.out.println("INSERTING DATA TO DB");
            client.insertToDB();
            System.out.println("DATA INSERTED BACK TO DB");
            client.closeConnection();
        } else {
            System.out.println("Invalid arguments!\nRun without arguments for recalculation or with two arguments for summary");
        }

    }
}
