import java.sql.Timestamp;
import java.util.Objects;

/**
 * Klic pod kterym se ukladaji zaznamy v jobech a dnech
 *
 * @author Petr Holicky
 */
public class Key {
    /* casova znacka */
    private final Timestamp data_time;
    /* id zarizeni */
    private final int id;

    /**
     * konstruktor
     * inicializuje data_time a id na zadane hodnoty
     * @param x casova znacka
     * @param y id
     */
    public Key(Timestamp x, int y) {
        this.data_time = x;
        this.id = y;
    }

    /**
     * testuje zda tento klic je pred klicem v parametru
     * @param key klic
     * @return true -> je pred, false -> neni pred nebo neni stejne id
     */
    public boolean before(Key key) {
        if(id != key.id) return false;
        return this.data_time.before(key.data_time);
    }

    /**
     * testuje zda tento klic je po klici v parametru
     * @param key klic
     * @return true -> je po, false -> neni po nebo neni stejne id
     */
    public boolean after(Key key) {
        if(id != key.id) return false;
        return this.data_time.after(key.data_time);
    }

    /**
     * urcuje v jakem formatu se klic vypisuje
     * @return data_time : id
     */
    @Override
    public String toString() {
        return data_time.toString()+" : "+id;
    }

    /**
     * testuje zda se klic rovna predanemu parametru
     * @param o object (jiny klic)
     * @return true -> rovna se, false -> nerovna se
     */
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof Key)) return false;
        if (((Key) o).data_time.compareTo(data_time) != 0) return false;
        if (((Key) o).id != id) return false;
        return true;
    }

    /**
     * vypocte hash pod kterym se klic uklada
     * @return hash
     */
    @Override
    public int hashCode() {
        return Objects.hash(data_time, id);
    }

    /**
     * vrati data_time
     * @return data_time
     */
    public Timestamp getData_time() {
        return data_time;
    }

    /**
     * vrati id
     * @return id
     */
    public int getId() {
        return id;
    }
}
